/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.kernel.axiom;

import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.transport.Communication;

import java.util.List;

/**
 * Brillien represents the probelms of real world as a free communications' flow. Components have 3 roles basically: start, receive communication and alter during communications decribed by interfaces Aspirer, Sensor and Progressor.
 * The Presence interface is the subinterface of the previous 3. All Brillien object must implement this interface, or in other words, every components in Brillien is a Presence instance. So presence and component are synonyms.
 * Every component has a unique name a flow and a presenceManager.
 */
public interface Presence<C extends Communication, R> extends Aspirer<C, R>, Sensor<C>, Progressor {

    /**
     * Gets the name of the Presence instance
     */
    String                  getName();

    /**
     * Gets the full name of the Presence instance
     */
    String                  getFullName();

    /**
     * Sets the name of the Presence instance
     */
    void                    setName( String name );

    /**
     * Retrieves the timeUnit value of the current callable activity
     */
    long                    getAspirationTimeMeasure();

    /**
     * Sets the timeUnit value of the current callable activity. Must be set right after instantiation
     */
    void                    setAspirationTimeMeasure(long timeout);

    /**
     * Tells whether the Aspirer is executed periodically or not.
     */
    boolean                 isAspirationPeriodical();

    /**
     * Sets the periodical execution of the Presence. Must be called before the activation process.
     */
    void                    setAspirationPeriodical( boolean periodical );

    /**
     * Invoked when a timeout service was started and timeout has been reached
     */
    void                    aspirationTimeIsUp();

    /**
     * Resets the presence to an initial state
     */
    void                    reset();

    /**
     * Retrieves whether the messageprocessor with the given name has a return value
     * @param messageProcessorName
     * @throws BrillienException if no annotation is present in the parameter list or messageProcessor doesn't exists)
     */
    boolean                 hasMessageProcessorReturnValue( String messageProcessorName ) throws BrillienException;

    /**
     * Retrieves the name of the given messageprocessor if the parameters are annotated with @P
     * @param messageProcessorName name of the messageProcessor
     * @return parameters' names
     * @throws BrillienException if no annotation is present in the parameter list or messageProcessor doesn't exists)
     */
    List<String>            getMessageProcessorParameterNames( String messageProcessorName ) throws BrillienException;

    /**
     * Retrieves the formal parameter list's types of the given messageprocessor
     * @param messageProcessorName name of the messageProcessor
     * @return parameters' types
     * @throws BrillienException if any error occurs  
     */
    List<String>            getMessageProcessorParameterTypes( String messageProcessorName ) throws BrillienException;

}
