/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.kernel.axiom;

import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.component.State;
import com.vii.brillien.kernel.axiom.transport.Communication;

/**
 * Brillien represents the probelms of real world as a free communications' flow. Components have 3 roles basically: start, receive communication and alter during communications
 * This interface define all receptive behaviors which belongs to receiving part of its roles.
 * A component should define and provide information about accepted message names, parameteres, types,
 * synchronous or asynchronous call possibilities. This interface define the message-receiver functionality of a component.
 * A Sensor object could contain an InputDivision object to validate the incoming messages.
 * This is a decision point before procesing any incomping messages. Invalid input message are not delegeted to the component. 
 */
public interface Sensor<C extends Communication> {

    /**
     * Special method invoked by Brillien if an error message has been received.
     * @param message error message received
     * @throws BrillienException in case of unhandled error
     */
    Object              errorReceived( C message ) throws BrillienException;

    /**
     * Notification method about an incoming text-based message addressed to this Sensor.
     * @param message message object received
     * @throws BrillienException Exception thrown in a problematic situation
     */
    Object              communicationReceived( C message  ) throws BrillienException;

    /**
     * Makes the object active
     */
    void                activateMessaging() throws BrillienException;

    /**
     * Makes the object passive. Every unanswered message should be thrown away.
     */
    void                passivateMessaging() throws BrillienException;


    /**
     * Invoked when this entity has been registered as a listener to other entity's state change events
     */
    void                presenseChangedState(String entityName, State oldState, State newState ) throws BrillienException;

}
